package ch.phoenix.oss.quarkus.commons.audit;

import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import jakarta.persistence.Column;
import jakarta.persistence.MappedSuperclass;
import java.time.Instant;
import org.hibernate.envers.NotAudited;

/**
 * The goal of this class is to have on the entity itself the exact same
 * timestamps as the ones from revisions generated by envers. Because of that,
 * we can't use @CreationTimestamp and @UpdateTimestamp, as those timestamp values
 * are managed by different Hibernate classes, so the generated values will drift.
 * <p>
 * Manually setting these values to match envers revisions would be error-prone,
 * verbose and tedious. So, the recommendation is to implement triggers on the
 * audit tables which will update the main entity whenever a revision is created.
 * An example of how to do that can be found in this module's integration tests.
 */
@MappedSuperclass
public abstract class AuditedPanacheEntityBase extends PanacheEntityBase {

    @NotAudited
    @Column(updatable = false)
    public Instant createdAt;

    @NotAudited
    @Column(updatable = false)
    public Instant lastUpdatedAt;
}
