package ch.phoenix.oss.quarkus.commons.audit;

import ch.phoenix.oss.quarkus.commons.tracing.TracingService;
import io.opentelemetry.instrumentation.annotations.WithSpan;
import io.quarkus.arc.DefaultBean;
import io.quarkus.arc.Unremovable;
import io.quarkus.logging.Log;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import java.net.InetAddress;
import java.net.UnknownHostException;

@Unremovable
@DefaultBean
@ApplicationScoped
class DefaultRevisionContextProvider implements RevisionContextProvider {

    private static final String UNKNOWN = "unknown";

    private final TracingService tracingService;

    @Inject
    DefaultRevisionContextProvider(TracingService tracingService) {
        this.tracingService = tracingService;
    }

    @Override
    @WithSpan
    public String getActor() {
        return tracingService.getActor();
    }

    @Override
    @WithSpan
    public String getTraceId() {
        return tracingService.getTraceId();
    }

    @Override
    @WithSpan
    public String getSpanId() {
        return tracingService.getSpanId();
    }

    @Override
    @WithSpan
    public String getRequestId() {
        return tracingService.getRequestId();
    }

    @Override
    @WithSpan
    public String getClientIp() {
        return tracingService.getClientIp();
    }

    @Override
    @WithSpan
    public String getHostName() {
        try {
            return InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException e) {
            Log.error("Unable to determine host name", e);
            return UNKNOWN;
        }
    }
}
