-- 1) Create or replace the trigger function
CREATE OR REPLACE FUNCTION trg_test_entity_aud_apply_rev()
    RETURNS TRIGGER
    LANGUAGE plpgsql
AS $func$
DECLARE
    ts TIMESTAMP;
BEGIN
    -- fetch the exact revision timestamp from revinfo
    SELECT r.timestamp
    INTO ts
    FROM revinfo r
    WHERE r.rev = NEW.rev;

    -- only set created_at once, when still NULL
    UPDATE test_entity
    SET created_at      = ts
    WHERE id = NEW.id
      AND created_at IS NULL;

    -- always bump last_updated_at
    UPDATE test_entity
    SET last_updated_at = ts
    WHERE id = NEW.id;

    RETURN NULL;  -- AFTER trigger ignores return value
END;
$func$;

-- 2) Drop any existing trigger, then attach the new one
DROP TRIGGER IF EXISTS trg_test_entity_aud_after_insert
    ON test_entity_aud;

CREATE TRIGGER trg_test_entity_aud_after_insert
    AFTER INSERT ON test_entity_aud
    FOR EACH ROW
EXECUTE FUNCTION trg_test_entity_aud_apply_rev();
