package ch.phoenix.oss.quarkus.commons.clock;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import io.quarkus.test.InjectMock;
import io.quarkus.test.junit.QuarkusTest;
import jakarta.inject.Inject;
import java.time.*;
import org.junit.jupiter.api.Test;

@QuarkusTest
class ClockServiceImplTest {

    @Inject
    ClockService clockService;

    @InjectMock
    Clock clock;

    @Test
    void instant() {
        var expected = Instant.ofEpochMilli(1729280640915L);
        when(clock.instant()).thenReturn(expected);

        assertThat(clockService.instant())
                .as("Instant should match expected value")
                .isEqualTo(expected);
    }

    @Test
    void currentTimeMillis() {
        var expected = 1729280640915L;
        when(clock.millis()).thenReturn(expected);

        assertThat(clockService.currentTimeMillis())
                .as("Instant should match expected value")
                .isEqualTo(expected);
    }

    @Test
    void localDate() {
        var instant = Instant.ofEpochMilli(1729280640915L);
        when(clock.instant()).thenReturn(instant);
        when(clock.getZone()).thenReturn(ZoneId.of("UTC"));
        assertThat(clockService.localDate())
                .as("LocalDate should match expected value")
                .isEqualTo(LocalDate.parse("2024-10-18"));
    }

    @Test
    void localDateTime() {
        var instant = Instant.ofEpochMilli(1729280640915L);
        when(clock.instant()).thenReturn(instant);
        when(clock.getZone()).thenReturn(ZoneId.of("UTC"));
        assertThat(clockService.localDateTime())
                .as("LocalDateTime should match expected value")
                .isEqualTo(LocalDateTime.parse("2024-10-18T19:44:00.915"));
    }

    @Test
    void zonedDateTime() {
        var instant = Instant.ofEpochMilli(1729280640915L);
        when(clock.instant()).thenReturn(instant);
        when(clock.getZone()).thenReturn(ZoneId.of("UTC"));
        assertThat(clockService.zonedDateTime())
                .as("ZonedDateTime should match expected value")
                .isEqualTo(ZonedDateTime.parse("2024-10-18T19:44:00.915Z[UTC]"));
    }
}
