package ch.phoenix.oss.quarkus.commons.digest;

import io.quarkus.arc.DefaultBean;
import jakarta.enterprise.context.ApplicationScoped;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

@DefaultBean
@ApplicationScoped
class MessageDigestServiceImpl implements MessageDigestService {

    private final MessageDigestConfiguration configuration;

    MessageDigestServiceImpl(MessageDigestConfiguration configuration) {
        this.configuration = configuration;
    }

    @Override
    public MessageDigest getInstance() {
        return getInstance(configuration.defaultAlgorithm());
    }

    @Override
    public MessageDigest getInstance(String algorithm) {
        try {
            return MessageDigest.getInstance(algorithm);
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("Unknown algorithm: " + algorithm, e);
        }
    }

    @Override
    public byte[] digest(String input) {
        return digest(input, StandardCharsets.UTF_8);
    }

    @Override
    public byte[] digest(String input, Charset charset) {
        return digest(input, charset, configuration.defaultAlgorithm());
    }

    @Override
    public byte[] digest(String input, Charset charset, String algorithm) {
        return digest(input, charset, getInstance(algorithm));
    }

    @Override
    public byte[] digest(String input, Charset charset, MessageDigest messageDigest) {
        var bytes = input.getBytes(StandardCharsets.UTF_8);
        return messageDigest.digest(bytes);
    }
}
