package ch.phoenix.oss.quarkus.commons.random;

import io.quarkus.arc.DefaultBean;
import jakarta.enterprise.context.ApplicationScoped;
import java.math.BigInteger;
import java.util.Random;

@DefaultBean
@ApplicationScoped
class RandomNumberGeneratorImpl implements RandomNumberGenerator {

    private final RandomProvider provider;

    RandomNumberGeneratorImpl(RandomProvider provider) {
        this.provider = provider;
    }

    @Override
    public int generateInt() {
        return provider.get().nextInt();
    }

    @Override
    public int generateInt(int bound) {
        return provider.get().nextInt(bound);
    }

    @Override
    public int generateInt(int origin, int bound) {
        return provider.get().nextInt(origin, bound);
    }

    @Override
    public long generateLong() {
        return provider.get().nextLong();
    }

    @Override
    public long generateLong(long bound) {
        return provider.get().nextLong(bound);
    }

    @Override
    public long generateLong(long origin, long bound) {
        return provider.get().nextLong(origin, bound);
    }

    @Override
    public float generateFloat() {
        return provider.get().nextFloat();
    }

    @Override
    public float generateFloat(float bound) {
        return provider.get().nextFloat(bound);
    }

    @Override
    public float generateFloat(float origin, float bound) {
        return provider.get().nextFloat(origin, bound);
    }

    @Override
    public double generateDouble() {
        return provider.get().nextDouble();
    }

    @Override
    public double generateDouble(double bound) {
        return provider.get().nextDouble(bound);
    }

    @Override
    public double generateDouble(double origin, double bound) {
        return provider.get().nextDouble(origin, bound);
    }

    @Override
    public BigInteger generateBigInteger(int numBits) {
        return new BigInteger(numBits, provider.get());
    }

    @Override
    public BigInteger generateBigInteger(int numBits, Random random) {
        return new BigInteger(numBits, random);
    }
}
