package ch.phoenix.oss.quarkus.commons.random;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.mockito.InjectSpy;
import jakarta.inject.Inject;
import java.math.BigInteger;
import java.util.Random;
import org.junit.jupiter.api.Test;

@QuarkusTest
class RandomNumberGeneratorImplTest {

    @Inject
    RandomNumberGeneratorImpl generator;

    @InjectSpy
    RandomProvider provider;

    @Test
    void generateInt() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateInt();

        assertThat(actual).as("Generated int should match expected value").isEqualTo(-1305521323);
    }

    @Test
    void generateIntWithBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateInt(40);

        assertThat(actual).as("Generated int should match expected value").isEqualTo(26);
    }

    @Test
    void generateIntWithOriginAndBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateInt(1, 40);

        assertThat(actual).as("Generated int should match expected value").isEqualTo(17);
    }

    @Test
    void generateLong() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateLong();

        assertThat(actual).as("Generated long should match expected value").isEqualTo(-5607171386684657918L);
    }

    @Test
    void generateLongWithBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateLong(40L);

        assertThat(actual).as("Generated long should match expected value").isEqualTo(9L);
    }

    @Test
    void generateLongWithOriginAndBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateLong(1L, 40L);

        assertThat(actual).as("Generated long should match expected value").isEqualTo(23L);
    }

    @Test
    void generateFloat() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateFloat();

        assertThat(actual).as("Generated float should match expected value").isEqualTo(0.6960346f);
    }

    @Test
    void generateFloatWithBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateFloat(0.99f);

        assertThat(actual).as("Generated float should match expected value").isEqualTo(0.6890743f);
    }

    @Test
    void generateFloatWithOriginAndBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateFloat(0.01f, 0.99f);

        assertThat(actual).as("Generated float should match expected value").isEqualTo(0.69211394f);
    }

    @Test
    void generateDouble() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateDouble();

        assertThat(actual).as("Generated double should match expected value").isEqualTo(0.6960346394874213d);
    }

    @Test
    void generateDoubleWithBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateDouble(0.99d);

        assertThat(actual).as("Generated double should match expected value").isEqualTo(0.6890742930925471d);
    }

    @Test
    void generateDoubleWithOriginAndBound() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateDouble(0.01d, 0.99d);

        assertThat(actual).as("Generated double should match expected value").isEqualTo(0.6921139466976729d);
    }

    @Test
    void generateBigInteger() {
        when(provider.get()).thenReturn(getRandom());

        var actual = generator.generateBigInteger(40);

        assertThat(actual)
                .as("Generated BigInteger should match expected value")
                .isEqualTo(BigInteger.valueOf(366467854850L));
    }

    @Test
    void generateBigIntegerWithRandomInstance() {
        var actual = generator.generateBigInteger(40, getRandom());

        assertThat(actual)
                .as("Generated BigInteger should match expected value")
                .isEqualTo(BigInteger.valueOf(366467854850L));
    }

    private static Random getRandom() {
        return new Random(24353L);
    }
}
