package ch.phoenix.oss.quarkus.commons.tracing;

import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.ArgumentMatchers.startsWith;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;

import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;
import io.quarkus.test.junit.mockito.InjectSpy;
import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import org.junit.jupiter.api.Test;

@QuarkusTest
@TestProfile(Test2Profile.class)
class RawPathTest {

    @InjectSpy
    TracingService tracingService;

    @Test
    void getWithRawPath() {
        var route = "/no-leading-and-trailing/{param}/{param2}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verify(tracingService).trace("actor", "anonymous");
        verify(tracingService).trace("request.method", "GET");
        verify(tracingService).trace("request.route", route);
        verify(tracingService).trace("request.path.params.param", "1");
        verify(tracingService).trace("request.path.params.param2", "2");
        verify(tracingService).trace("request.path.raw", "/no-leading-and-trailing/1/2");
        verify(tracingService).trace("request.headers.accept", "text/plain");
        verify(tracingService).trace("request.headers.accept-encoding", "gzip,deflate");
        verify(tracingService).trace("request.headers.connection", "Keep-Alive");
        verify(tracingService).trace(eq("request.headers.host"), startsWith("localhost:"));
        verify(tracingService).trace(eq("request.headers.user-agent"), startsWith("Apache-HttpClient"));
        verify(tracingService).trace("request.client.ip", "127.0.0.1");
        verifyNoMoreInteractions(tracingService);
    }
}
