package ch.phoenix.oss.quarkus.commons.tracing;

import static org.assertj.core.api.Assertions.assertThat;

import io.opentelemetry.api.trace.Span;
import io.opentelemetry.instrumentation.annotations.WithSpan;
import io.quarkus.test.junit.QuarkusTest;
import jakarta.inject.Inject;
import org.jboss.logging.MDC;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

@QuarkusTest
class TracingServiceImplTest {

    @Inject
    TracingService tracingService;

    @Inject
    Span span;

    @BeforeEach
    void setUp() {
        MDC.clear();
    }

    @Test
    void getActor() {
        tracingService.trace("actor", "abc");
        assertThat(tracingService.getActor())
                .as("Actor should match expected value")
                .isEqualTo("abc");
    }

    @Test
    void getRequestPathRaw() {
        tracingService.trace("request.path.raw", "/foo/bar");
        assertThat(tracingService.getRequestPathRaw())
                .as("Request Path Raw should match expected value")
                .isEqualTo("/foo/bar");
    }

    @Test
    void getRequestMethod() {
        tracingService.trace("request.method", "GET");
        assertThat(tracingService.getRequestMethod())
                .as("Request Method should match expected value")
                .isEqualTo("GET");
    }

    @Test
    void getRequestId() {
        tracingService.trace("request.headers.x-request-id", "ba458367-bfeb-46ba-87da-50b9343be8f9");
        assertThat(tracingService.getRequestId())
                .as("Request Id should match expected value")
                .isEqualTo("ba458367-bfeb-46ba-87da-50b9343be8f9");
    }

    @Test
    @WithSpan
    void getTraceId() {
        assertThat(tracingService.getTraceId())
                .as("Request Trace Id should match expected value")
                .isEqualTo(span.getSpanContext().getTraceId());
    }

    @Test
    @WithSpan
    void getSpanId() {
        assertThat(tracingService.getSpanId())
                .as("Request Span Id should match expected value")
                .isEqualTo(span.getSpanContext().getSpanId());
    }

    @Test
    void getClientIp() {
        tracingService.trace("request.client.ip", "127.0.0.1");
        assertThat(tracingService.getClientIp())
                .as("Request Client Iü should match expected value")
                .isEqualTo("127.0.0.1");
    }

    @Test
    void getSchedulerJob() {
        tracingService.trace("scheduler.job.name", "scheduler/abc");
        assertThat(tracingService.getSchedulerJob())
                .as("Scheduler Job Name should match expected value")
                .isEqualTo("scheduler/abc");
    }

    @Test
    void clearAll() {
        tracingService.trace("aaa", "bbb");
        assertThat(MDC.get("aaa")).isEqualTo("bbb");
        tracingService.clearAll();
        assertThat(MDC.get("aaa")).isNull();
    }
}
